function C=A_W(n,law)
% computes the constant A_W for a sample size n and a resampling distribution given by 'law'
% possible values for the string law:
% 'Rad'    : Rad(1/2) weights
% 'Rad(p)' : Rad(p) weights
% 'Efr'    : Efron(n) weights
% 'Efr(M)' : Efron(M) weights
% 'Rho'    : Rho(n/2) weights (or (n-1)/2 if n is odd)
% 'Rho(q)' : Rho(q) weights
% 'Loo'    : Rho(n-1) weights
% 'Ber'    : Ber(1/2) weights ( = 0 or 2 with equal probability ) = Rad(1/2) +1
% 'Ber(p)' : Ber(p) weights
% 'VFCV(V)' : V-fold cross-validation weights (the last argument, B, is not used)

switch (law(1:3))
    case {'Rad','Ber'}
	if length(law)==3
		param=1/2;
	else
		param=str2double(law(5:(length(law)-1)));
    end

    if (param<1 && param>0)
        C=2*(1-param)*(1-(1/n));
    else
        C=NaN;
    end

    case 'Efr'
	if length(law)==3
		param=n;
	else
		param=str2double(law(5:(length(law)-1)));
	end
	if param<=n
		C=2*(1-(1/n))^param;
    else
        k=ceil(param/n)-1;
        C=sum(2*(1-((n/param)*(0:k))).*binopdf(0:k,param,1/n));
	end

    case 'Rho'
	if length(law)==3
		param=floor(n/2);
	else
		param=min(n,str2double(law(5:(length(law)-1))));
	end

	C = 2*(1 - (param/n));

    case 'Loo'
	C = 2/n;

    case {'VFC'}
        param=str2double(law(6:(length(law)-1)));
        if (param>1 && param<=n && param==floor(param))
            list=1+floor((n/param)*(0:param));
            card=list(2:end)-list(1:end-1);
            C=( sum( 2 * (card/n).*(1-(card/n)) ) ) / (param-1);
        else
            C=NaN;
        end

    otherwise
	C=NaN;

end%switch (law(1:3))

