function [FP,TP,FN,TN]=Power_level(thresh,Emp,mean_vec,type)
%
% This function computes the number of false/true positive/negative for a given threshold
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
% IN : 
% 
%	'thresh' : value of the threshold
%	'Emp' : vector of the empirical means
%	'mean_vec' : string describing the vector of the true mean (see function mean_vector.m)
%   'type' : type of experiment (function phi);
%                currently available:
%                   'sup_one_sided'
%                   'sup_two_sided'
%	
%  OUT : 
%
%   'FP' : number of false positives
%   'TP' : number of true positives
%   'FN' : number of false negatives
%   'TN' : number of true negatives

%%% True mean vector
mu=mean_vector(mean_vec,numel(Emp));

%%% Vector of "positives" (which null hypotheses are rejected) : vec_pos
%%% Vector of "true positives" (which null hypotheses are false) : vec_tru

switch type
    case 'sup_one_sided'
        vec_pos=(Emp > thresh);
        vec_tru=(mu > 0);
    case 'sup_two_sided'
        vec_pos=(abs(Emp) > thresh);
        vec_tru=(abs(mu) > 0);
    otherwise
        vec_pos=zeros(1,numel(Emp));
        vec_tru=zeros(1,numel(Emp));
end

FP=sum(vec_pos.*(1-vec_tru));
TP=sum(vec_pos.*(vec_tru));
FN=sum((1-vec_pos).*(vec_tru));
TN=sum((1-vec_pos).*(1-vec_tru));