% Script for the simulation study of ``Some non-asymptotic results on resampling in high dimension, II: multiple tests'' by Sylvain Arlot, Gilles Blanchard and Etienne Roquain
% This script generates all the figures of the paper
% Tunable parameters (e.g. printing format, number of samples considered)
% can be changed on the first lines of the code
%
% Copyright (C) 2009 Sylvain Arlot, Gilles Blanchard and Etienne Roquain
%
%   This program is free software: you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation, either version 3 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License
%   along with this program.  If not, see <http://www.gnu.org/licenses/>
% 

%%%%%%%%%%%%%%%%%%%% Tunable parameters

%%%% Printing directory
print_dir = ['tmp/'];

%%%% Printing format
z_error_bars=0; % put error bars on the figures (1 = yes ; 0 = no)

z_eps=1;% print figures as eps
z_pdf=0;% print figures as pdf

%%%% Which figures should be print ? 
%%%% Which computations should be done (again) ?
z_fig1=1;
z_fig1_calc=1;%%% 1 = make again the computations and save ; 0 = load previouly made computations

z_fig2=1;
z_fig2_calc=1;%%% 1 = make again the computations and save ; 0 = load previouly made computations

z_fig3=1;
z_fig3_calc=1;%%% 1 = make again the computations and save ; 0 = load previouly made computations


%%% Parameters used for the paper, but may be changed (number of samples
%%% considered, etc.)
fig1_nb_repet=250;%%%% number of samples considered for Fig 1 Right

fig2_nb_repet=250;%%%% number of samples considered for Fig 2

fig3_nb_repet=250;%%%% number of samples considered for Fig 3
fig3_bandwidth=30;%%%% constant bandwidth considered for Fig 3
fig3_nb_obs=100;%%%% number of observations considered for Fig 3

N=0;%%% useful for performing several simulations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Do not change anything below this line, use abr08_main.m instead
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Constant parameters

%%% Generation of data
nb_obs_fig1_2=1000;
dim=128*128;
data_type='gauss_texture';
filter='exp(d^2)';
sigma_gener=1;
range_vect=(5:5:30);%%% only useful for Fig 3

%%% Analysis of data
level=0.05;
type='sup_two_sided';
data_law='Gaussian';
sigma_known=sigma_gener;
weights='Rad';
nb_mcmc=1000;
deltacb=0.1;
deltaq=0.1;
alpha0_mult=0.9;

%%%%% Precompute useful constants
if (z_fig1 || z_fig2)
    nb_obs=nb_obs_fig1_2;
    BW=B_W(nb_obs,weights);
    CW=C_W(nb_obs,weights);
    gamma=1-2*binoinv(deltaq*level*alpha0_mult/2,nb_obs,0.5)/(nb_obs);
end

%%%% Other parameters
nb_simu=1000; % number of data samples generated for estimating the 'ideal' threshold
fontsize=14; %Font size for the graphs
linewidth=2; %Line width for the graphs

%%%%%%%%%%%%%%%%%%%% Figure 1 : unspecified alternative, translation-invariant procedure
i_fig=0;

if z_fig1
	mean_vec='zero';
    nb_obs=nb_obs_fig1_2;
	
	
%%%%%%%%% Fig 1 Left : example of a 128*128 pixels image
	bandwidth = 18;
	Y=gener_data(1,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'mean',mean_vec,'sigma',sigma_gener);
	h=visu_data(Y);
    i_fig=h+1;
    if z_eps
        print('-f','-r600','-depsc2',[print_dir 'Fig1_Left_' data_type '_' num2str(bandwidth) '.eps']);
    end
    if z_pdf
        print('-f','-r600','-dpdf',[print_dir 'Fig1_Left_' data_type '_' num2str(bandwidth) '.pdf']);
    end

%%%%%%%%% Fig 1 Right : average thresholds 
	if z_fig1_calc
        clear result;
        thresholds_list={'bonf','conc','conc_bonf','quant_bonf','quant_conc','ideal','single'};
    %%%%% Make the computations
        fprintf('Starting computations for Figure 1\n');
        tic;
        bandwidth_vect=(0:2:40);
        %%% The two loops go down in order to preallocate memory in struct 'result'
        for i_band=numel(bandwidth_vect):-1:1
            bandwidth=bandwidth_vect(i_band);
            fprintf('Bandwidth = %d : Computing ideal threshold \n',bandwidth);
            %Compute ideal threshold
            ideal=ideal_threshold(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'sigma',sigma_gener,'nb_simu',nb_simu,'type',type);
% % %             %result_ideal(i_band)=ideal_threshold(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'sigma',sigma_gener,'nb_simu',nb_simu,'type',type);
            for i_sample=fig1_nb_repet:-1:1
%                fprintf('Sample number %d over %d \n',fig1_nb_repet-i_sample+1,fig1_nb_repet);
                fprintf('N=%d ; Bandwidth=%d ; Sample number %d over %d \n',N,bandwidth,fig1_nb_repet-i_sample+1,fig1_nb_repet);
                Y=gener_data(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'mean',mean_vec,'sigma',sigma_gener);
                res=thresholds(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                    'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma,...
                    'thresholds_list',thresholds_list);
                res.bandwidth=bandwidth;
                res.ideal=ideal;
                
                %%% Computes empirical mean
                Empmat = mean(Y,2);
                %%% Record number of true/false positives/negatives for each threshold
                for i_thr=1:numel(thresholds_list)
                    [res.([ thresholds_list{i_thr} '_FP']) , res.([ thresholds_list{i_thr} '_TP']) , res.([ thresholds_list{i_thr} '_FN']) , ...
                        res.([ thresholds_list{i_thr} '_TN']) ]=Power_level(res.(thresholds_list{i_thr}),Empmat',mean_vec,type);
                end%for i_thr=1:numel(thresholds_list)

                result(i_band,i_sample)=res;
                
                toc;
            end%for i_sample=1:fig1_nb_repet
        end%for bandwidth=(0:2:40)
        save([print_dir 'Fig1_data.mat'],'result');
%%%        save([print_dir 'Fig1_ideal.mat'],'result_ideal','bandwidth_vect');
	else
        load([print_dir 'Fig1_data.mat'],'result');
%%%        load([print_dir 'Fig1_ideal.mat'],'result_ideal','bandwidth_vect');
    end%if z_fig1_calc
    
    %%%%% Analyse data
    %%% Recover list of bandwidth considered
        bandwidth_list=[result.bandwidth];
        bandwidth_vect=unique(bandwidth_list);
        
% % %         %%% First, treat the case of the ideal threshold
% % %         %ideal_mean=zeros(size(bandwidth_vect0));
% % %         %ideal_err=zeros(size(bandwidth_vect0));
% % %         %for i_b=1:numel(bandwidth_vect0)
% % %         %   ideal_mean(i_b)=result_ideal_fig1(bandwidth_vect==bandwidth_vect0(i_b));
% % %         %end
% % %         bandwidth_vect=bandwidth_vect0;
        

        thresholds_list={'bonf','conc','conc_bonf','quant_bonf','quant_conc','ideal','single'};
        thresholds_names={'bonf','conc','conc\wedge bonf','quant+bonf','quant+conc','ideal','single'};
        thresholds_line={'k-','k:','k--','k--x','k:x','k-.','k-v'};

        
        figure(i_fig);
        newplot;
        hold on;
        legend_str='';
        set(gca,'FontSize',fontsize);
        for i_thr=1:numel(thresholds_list)
            s=thresholds_list{i_thr};
            %%% Recover values of the thresholds
                eval([ s '_list = [result.' s '];']);
            %%% Preallocate memory
                eval([ s '_mean = zeros(size(bandwidth_vect));']);
                eval([ s '_err = zeros(size(bandwidth_vect));']);
                for i_b=1:numel(bandwidth_vect)
                    band=bandwidth_vect(i_b);
                    vec=eval([ s '_list ( bandwidth_list == band )']);
                    eval([ s '_mean (i_b) = mean(vec);']);
                    eval([ s '_err (i_b) = std(vec)/sqrt(numel(vec));']);
                end
            %%% Plot the data
            if z_error_bars
            %%% with error bars
                eval(['errorbar(bandwidth_vect, ' s '_mean , ' s '_err , ''' thresholds_line{i_thr} ''' , ''Linewidth'', linewidth )']);
            else
            %%% without error bars
                eval(['plot(bandwidth_vect, ' s '_mean , ''' thresholds_line{i_thr} ''' , ''Linewidth'', linewidth )']);
            end%if z_error_bars

            legend_str=[legend_str ' thresholds_names{ ' num2str(i_thr) ' } , '];
        end
        xlabel('Convolution kernel width (pixels)');
        ylabel('Av. threshold');
        title({['Gaussian kernel convolution, K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
        eval(['legend(' legend_str ' ''Location'',''NorthEast'' )']);
        hold off;

    
    %%%%% Plot data

    if z_eps
        print('-f','-r600','-depsc2',[print_dir 'Fig1_Right_' data_type '.eps']);
    end
    if z_pdf
        print('-f','-r600','-dpdf',[print_dir 'Fig1_Right_' data_type '.pdf']);
    end


end%z_fig1

%%%%%%%%%%%%%%%%%%%% Figure 2 : specific alternative (linear increasing of
%%%%%%%%%%%%%%%%%%%% the mean)
i_fig=i_fig+1;

if z_fig2
	mean_vec='linear';
    nb_obs=nb_obs_fig1_2;
	
	%%%%%%%% Make computations
	if z_fig2_calc
        clear result;
        thresholds_list_sd={'sd_quant_uncent','sd_bonf','sd_quant_bonf'};
        thresholds_list_stepone={'quant_uncent','bonf','quant_bonf','single','ideal','conc','conc_bonf','quant_conc'};%%% the last three ones will not be plotted, but can be useful to recover Fig 1
%        thresholds_list=union(thresholds_list_sd,thresholds_list_stepone);
        
        fprintf('Starting computations for Figure 2\n');
        tic;
        bandwidth_vect=(0:2:40);
        %%% The two loops go down in order to preallocate memory in struct 'result'
        for i_band=numel(bandwidth_vect):-1:1
            bandwidth=bandwidth_vect(i_band);
            fprintf('Bandwidth = %d : Computing ideal threshold \n',bandwidth);
            %Compute ideal threshold
            ideal=ideal_threshold(nb_obs,dim,'mean',mean_vec,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'sigma',sigma_gener,'nb_simu',nb_simu,'type',type);
            for i_sample=fig2_nb_repet:-1:1
%                fprintf('N=%d ; Figure 2 ; Bandwidth = %d ; Sample number %d over %d \n',N,bandwidth,fig2_nb_repet-i_sample+1,fig2_nb_repet);
                fprintf('Figure 2 ; Bandwidth = %d ; Sample number %d over %d \n',bandwidth,fig2_nb_repet-i_sample+1,fig2_nb_repet);
                Y=gener_data(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'mean',mean_vec,'sigma',sigma_gener);

                %%%%% Start with the one-step thresholds (will also be used for
                %%%%% step-down thresholds)
                
                res_onestep=thresholds(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                    'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma);
                res_onestep.ideal=ideal;%%%
                res_onestep.bandwidth=bandwidth;
                
                %%% Computes empirical mean
                Empmat = mean(Y,2);
                %%% Record number of true/false positives/negatives for each threshold
                for i_thr=1:numel(thresholds_list_stepone)
                    [res_onestep.([ thresholds_list_stepone{i_thr} '_FP']) , res_onestep.([ thresholds_list_stepone{i_thr} '_TP']) , res_onestep.([ thresholds_list_stepone{i_thr} '_FN']) , ...
                        res_onestep.([ thresholds_list_stepone{i_thr} '_TN']) ]=Power_level(res_onestep.(thresholds_list_stepone{i_thr}),Empmat',mean_vec,type);
                    res_onestep.([ thresholds_list_stepone{i_thr} '_power'])=(res_onestep.([ thresholds_list_stepone{i_thr} '_TP']))/...
                        ((res_onestep.([ thresholds_list_stepone{i_thr} '_TP']))+(res_onestep.([ thresholds_list_stepone{i_thr} '_FN'])));
                end%for i_thr=1:numel(thresholds_list)

                %%%%% Now compute the step-down thresholds 
                res_sd=thresholds_sd(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                    'thresholds_list',thresholds_list_sd,'results_stepone',res_onestep,...
                    'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma);

                %%% Store the computed thresholds in the struct 'res'
                res=res_onestep;
                for i_thr=1:numel(thresholds_list_sd)
                    res.(thresholds_list_sd{i_thr})=res_sd.(thresholds_list_sd{i_thr});
                end

                
                %%% Record number of true/false positives/negatives for
                %%% each threshold at each stage of the algorithm
                for i_thr=1:numel(thresholds_list_sd)
                    %%% Retrieve the list of thresholds
                    th_v=res.(thresholds_list_sd{i_thr});
                    %%% Then, for each element of the list, evaluate the
                    %%% number of true/false positives/negatives
                    n_v=numel(th_v);
                    FP_v=zeros(1,n_v);
                    TP_v=zeros(1,n_v);
                    FN_v=zeros(1,n_v);
                    TN_v=zeros(1,n_v);
                    for i_v=1:n_v
                        [FP_v(i_v) TP_v(i_v) FN_v(i_v) TN_v(i_v)]=Power_level(th_v(i_v),Empmat',mean_vec,type);
                    end%for i_v=1:n_v
                    %%% Store the four vectors FP_v, TP_v, FN_v, TN_v in
                    %%% the struct res_sd
                    res.([ thresholds_list_sd{i_thr} '_FP'])=FP_v;
                    res.([ thresholds_list_sd{i_thr} '_TP'])=TP_v;
                    res.([ thresholds_list_sd{i_thr} '_FN'])=FN_v;
                    res.([ thresholds_list_sd{i_thr} '_TN'])=TN_v;
                    res.([ thresholds_list_sd{i_thr} '_power'])=(TP_v ./ (TP_v+FN_v));
                    
                end%for i_thr=1:numel(thresholds_list)

                
                result(i_band,i_sample)=res;
                
                toc;
            end%for i_sample=1:fig2_nb_repet
        end%for bandwidth=(0:2:40)
        save([print_dir 'Fig2_data.mat'],'result');
% % %        save([print_dir 'Fig2_ideal.mat'],'result_ideal','bandwidth_vect');
	else
        load([print_dir 'Fig2_data.mat'],'result');
% % %        load([print_dir 'Fig2_ideal.mat'],'result_ideal','bandwidth_vect');
    end%if z_fig2_calc
    
    %%%%% Analyse data
    %%% Recover list of bandwidth considered
        bandwidth_list=[result.bandwidth];
        bandwidth_vect=unique(bandwidth_list);
        
% % %         %%% First, treat the case of the ideal threshold
% % %         %ideal_mean=zeros(size(bandwidth_vect0));
% % %         %ideal_err=zeros(size(bandwidth_vect0));
% % %         %for i_b=1:numel(bandwidth_vect0)
% % %         %   ideal_mean(i_b)=result_ideal(bandwidth_vect==bandwidth_vect0(i_b));
% % %         %end
% % %         bandwidth_vect=bandwidth_vect0;

        thresholds_list={'quant_uncent','bonf','sd_bonf','quant_bonf','sd_quant_bonf','ideal','sd_quant_uncent','single'};
        thresholds_names={'quant. uncent.','bonf','holm','quant+bonf','s.d. quant+bonf','ideal','s.d. quant uncent','single'};
        thresholds_line={'k:','k-','k-s','k--x','k--s','k-.','k:s','k-v'};        
        
        %%%% Figure 2 Left : average value of the threshold
        figure(i_fig);
        newplot;
        hold on;
        legend_str='';
        set(gca,'FontSize',fontsize);
        for i_thr=1:numel(thresholds_list)
        %%% Recover values of the thresholds (taking the final value for
        %%% step-down methods), the number of steps needed and the power
            s=thresholds_list{i_thr};
            
            %%% Preallocate memory
            eval([ s '_list = zeros(1,numel(bandwidth_list));']);
            eval([ s '_nb_step_list = zeros(1,numel(bandwidth_list));']);
            eval([ s '_power_list = zeros(1,numel(bandwidth_list));']);
            for i_bl=1:numel(bandwidth_list)
                %%% values of the thresholds
                eval([ s '_list(i_bl) = result(i_bl).' s '(end);']);
                %%% number of steps (only interesting for step-down methods)
                eval([ s '_nb_step_list(i_bl) = numel(result(i_bl).' s ');']);
                %%% power of the thresholds
                eval([ s '_power_list(i_bl) = result(i_bl).' s '_power(end);']);
            end%for i_bl=1:numel(bandwidth_list)

            %%% Preallocate memory
            eval([ s '_mean = zeros(size(bandwidth_vect));']);
            eval([ s '_err = zeros(size(bandwidth_vect));']);
            eval([ s '_power_mean = zeros(size(bandwidth_vect));']);
            eval([ s '_power_err = zeros(size(bandwidth_vect));']);
            for i_b=1:numel(bandwidth_vect)
                band=bandwidth_vect(i_b);
                %%% value of the threshold
                vec=eval([ s '_list ( bandwidth_list == band )']);
                eval([ s '_mean (i_b) = mean(vec);']);
                eval([ s '_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%% power of the threshold
                vec=eval([ s '_power_list ( bandwidth_list == band )']);
                eval([ s '_power_mean (i_b) = mean(vec);']);
                eval([ s '_power_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%% number of steps (only interesting for step-down methods)
                vec=eval([ s '_nb_step_list ( bandwidth_list == band )']);
                eval([ s '_nb_step_mean (i_b) = mean(vec);']);
                eval([ s '_nb_step_err (i_b) = std(vec)/sqrt(numel(vec));']);
            end
        %%% Plot the data
            if z_error_bars
            %%% with error bars
                eval(['errorbar(bandwidth_vect, ' s '_mean , ' s '_err , ''' thresholds_line{i_thr} ''' , ''Linewidth'', linewidth )']);
            else
            %%% without error bars
                eval(['plot(bandwidth_vect, ' s '_mean , ''' thresholds_line{i_thr} ''' , ''Linewidth'', linewidth )']);
            end%if z_error_bars

            legend_str=[legend_str ' ''' thresholds_names{i_thr} ''', '];
        end%for i_thr=1:numel(thresholds_list)
        xlabel('Convolution kernel width (pixels)');
        ylabel('Av. threshold');
        title({['Gaussian kernel convolution, K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
        eval(['legend(' legend_str ' ''Location'',''NorthEast'' )']);
        hold off;

    
    %%%%% Plot data

    if z_eps
        print('-f','-r600','-depsc2',[print_dir 'Fig2_Left_' data_type '.eps']);
    end
    if z_pdf
        print('-f','-r600','-dpdf',[print_dir 'Fig2_Left_' data_type '.pdf']);
    end

        thresholds_list={'quant_uncent','bonf','sd_bonf','quant_bonf','sd_quant_bonf','ideal','sd_quant_uncent'};
        thresholds_names={'quant. uncent.','bonf','holm','quant+bonf','s.d. quant+bonf','ideal','s.d. quant uncent'};
        thresholds_line={'k:','k-','k-s','k--x','k--s','k-.','k:s'};        

        %%%% Figure 2 Right : Plot the average value of the power
        i_fig=i_fig+1;
        figure(i_fig);
        newplot;
        hold on;
        legend_str='';
        set(gca,'FontSize',fontsize);
        for i_thr=1:numel(thresholds_list)
            s=thresholds_list{i_thr};
            if z_error_bars
            %%% with error bars
                eval(['errorbar(bandwidth_vect, ' s '_power_mean , ' s '_power_err , ''' thresholds_line{i_thr} ''' , ''Linewidth'', linewidth )']);
            else
            %%% without error bars
                eval(['plot(bandwidth_vect, ' s '_power_mean , ''' thresholds_line{i_thr} ''' , ''Linewidth'', linewidth )']);
            end%if z_error_bars

            legend_str=[legend_str ' ''' thresholds_names{i_thr} ''', '];
        end
        xlabel('Convolution kernel width (pixels)');
        ylabel('Power');
        title({['Gaussian kernel convolution, K=' num2str(sqrt(dim)) 'x' num2str(sqrt(dim)) ', n=' num2str(nb_obs) ', level ' num2str(level * 100) '%']});
        eval(['legend(' legend_str ' ''Location'',''SouthEast'' )']);
        hold off;

    
    %%%%% Plot data

    if z_eps
        print('-f','-r600','-depsc2',[print_dir 'Fig2_Right_' data_type '.eps']);
    end
    if z_pdf
        print('-f','-r600','-dpdf',[print_dir 'Fig2_Right_' data_type '.pdf']);
    end
end%if z_fig2

%%%%%%%%%%%%%%%%%%%% Figure 3 : specific alternative (exponential increasing of
%%%%%%%%%%%%%%%%%%%% the mean)
if z_fig3
	mean_vec='exponential';
    %%% The bandwidth is now fixed
    bandwidth=fig3_bandwidth;
    %%% The number of observations is specific for this Figure
    nb_obs=fig3_nb_obs;
	
    %%%%% Precompute useful constants
    BW=B_W(nb_obs,weights);
    CW=C_W(nb_obs,weights);
    gamma=1-2*binoinv(deltaq*level*alpha0_mult/2,nb_obs,0.5)/(nb_obs);

    %%%%%%%% Make computations
	if z_fig3_calc
        clear result;
        thresholds_list_sd={'sd_quant_uncent','hybrid'};
        thresholds_list_stepone={'quant_uncent','quant_bonf'};
        
        fprintf('Starting computations for Figure 3\n');
        tic;
        %%% The two loops go down in order to preallocate memory in struct 'result'
        for i_r=numel(range_vect):-1:1
            dynamic_range=range_vect(i_r);
             fprintf('Dynamic range = %d :\n',dynamic_range);
%              %Compute ideal threshold
%              ideal=ideal_threshold(nb_obs,dim,'mean',mean_vec,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'sigma',sigma_gener,'nb_simu',nb_simu,'type',type,'dynamic_range',dynamic_range);
            for i_sample=fig3_nb_repet:-1:1
%                fprintf('N=%d ; Figure 3 ; Dynamic range = %d ; Sample number %d over %d \n',N,dynamic_range,fig3_nb_repet-i_sample+1,fig3_nb_repet);
                fprintf('Figure 3 ; Dynamic range = %d ; Sample number %d over %d \n',dynamic_range,fig3_nb_repet-i_sample+1,fig3_nb_repet);
                Y=gener_data(nb_obs,dim,'data_type',data_type,'filter',filter,'bandwidth',bandwidth,'mean',mean_vec,'sigma',sigma_gener,'dynamic_range',dynamic_range);

                %%%%% Start with the one-step thresholds (will also be used for
                %%%%% step-down thresholds)
                
                res_onestep=thresholds(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                    'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma);
%                 res_onestep.ideal=ideal;
                res_onestep.dynamic_range=dynamic_range;
                
                %%% Computes empirical mean
                Empmat = mean(Y,2);
                %%% Record number of true/false positives/negatives for each threshold
                for i_thr=1:numel(thresholds_list_stepone)
                    [res_onestep.([ thresholds_list_stepone{i_thr} '_FP']) , res_onestep.([ thresholds_list_stepone{i_thr} '_TP']) , res_onestep.([ thresholds_list_stepone{i_thr} '_FN']) , ...
                        res_onestep.([ thresholds_list_stepone{i_thr} '_TN']) ]=Power_level(res_onestep.(thresholds_list_stepone{i_thr}),Empmat',mean_vec,type);
                    res_onestep.([ thresholds_list_stepone{i_thr} '_power'])=(res_onestep.([ thresholds_list_stepone{i_thr} '_TP']))/...
                        ((res_onestep.([ thresholds_list_stepone{i_thr} '_TP']))+(res_onestep.([ thresholds_list_stepone{i_thr} '_FN'])));
                end%for i_thr=1:numel(thresholds_list)

                %%%%% Now compute the step-down thresholds 
                res_sd=thresholds_sd(Y,'level',level,'type',type,'data_law',data_law,'sigma',sigma_known,'weights',weights,...
                    'thresholds_list',thresholds_list_sd,'results_stepone',res_onestep,...
                    'nb_mcmc',nb_mcmc,'deltacb',deltacb,'deltaq',deltaq,'alpha0_mult',alpha0_mult,'BW',BW,'CW',CW,'gamma',gamma);

                %%% Store the computed thresholds in the struct 'res'
                res=res_onestep;
                for i_thr=1:numel(thresholds_list_sd)
                    res.(thresholds_list_sd{i_thr})=res_sd.(thresholds_list_sd{i_thr});
                end

                
                %%% Record number of true/false positives/negatives for
                %%% each threshold at each stage of the algorithm
                for i_thr=1:numel(thresholds_list_sd)
                    %%% Retrieve the list of thresholds
                    th_v=res.(thresholds_list_sd{i_thr});
                    %%% Then, for each element of the list, evaluate the
                    %%% number of true/false positives/negatives
                    n_v=numel(th_v);
                    FP_v=zeros(1,n_v);
                    TP_v=zeros(1,n_v);
                    FN_v=zeros(1,n_v);
                    TN_v=zeros(1,n_v);
                    for i_v=1:n_v
                        [FP_v(i_v) TP_v(i_v) FN_v(i_v) TN_v(i_v)]=Power_level(th_v(i_v),Empmat',mean_vec,type);
                    end%for i_v=1:n_v
                    %%% Store the four vectors FP_v, TP_v, FN_v, TN_v in
                    %%% the struct res_sd
                    res.([ thresholds_list_sd{i_thr} '_FP'])=FP_v;
                    res.([ thresholds_list_sd{i_thr} '_TP'])=TP_v;
                    res.([ thresholds_list_sd{i_thr} '_FN'])=FN_v;
                    res.([ thresholds_list_sd{i_thr} '_TN'])=TN_v;
                    res.([ thresholds_list_sd{i_thr} '_power'])=(TP_v ./ (TP_v+FN_v));
                    
                end%for i_thr=1:numel(thresholds_list)

                
                result(i_r,i_sample)=res;
                
                toc;
            end%for i_sample=1:fig3_nb_repet
        end%for i_r=numel(range_vect):-1:1
        save([print_dir 'Fig3_data.mat'],'result');
	else
        load([print_dir 'Fig3_data.mat'],'result');
    end%if z_fig3_calc
    
    %%%%% Analyse data
    %%% Recover list of bandwidth considered
        range_list=[result.dynamic_range];
        range_vect=unique(range_list);

        thresholds_list={'sd_quant_uncent','hybrid'};
        
        %%%% Figure 3 Left : average number of iterations
        for i_thr=1:(numel(thresholds_list))
        %%% Recover values of the thresholds (taking the final value for
        %%% step-down methods), the number of steps needed and the power
            s=thresholds_list{i_thr};
        
            %%% Preallocate memory
            eval([ s '_list = zeros(1,numel(range_list));']);
            eval([ s '_nb_step_list = zeros(1,numel(range_list));']);
            eval([ s '_power_list = zeros(1,numel(range_list));']);
            eval([ s '_power1_list = zeros(1,numel(range_list));']);
            eval([ s '_power2_list = zeros(1,numel(range_list));']);
            eval([ s '_power3_list = zeros(1,numel(range_list));']);
            for i_bl=1:numel(range_list)
                %%% values of the thresholds
                eval([ s '_list(i_bl) = result(i_bl).' s '(end);']);
                %%% number of steps (only interesting for step-down methods)
                eval([ s '_nb_step_list(i_bl) = numel(result(i_bl).' s ');']);
                %%% power of the thresholds
                eval([ s '_power_list(i_bl) = result(i_bl).' s '_power(end);']);
                %%% power of the thresholds stopped at step 1
                eval([ s '_power1_list(i_bl) = result(i_bl).' s '_power(min(end,1));']);
                %%% power of the thresholds stopped at step 2
                eval([ s '_power2_list(i_bl) = result(i_bl).' s '_power(min(end,2));']);
                %%% power of the thresholds stopped at step 3
                eval([ s '_power3_list(i_bl) = result(i_bl).' s '_power(min(end,3));']);
            end%for i_bl=1:numel(bandwidth_list)

            %%% Preallocate memory
            eval([ s '_mean = zeros(size(range_vect));']);
            eval([ s '_err = zeros(size(range_vect));']);
            eval([ s '_nb_step_mean = zeros(size(range_vect));']);
            eval([ s '_nb_step_err = zeros(size(range_vect));']);
            eval([ s '_power_mean = zeros(size(range_vect));']);
            eval([ s '_power_err = zeros(size(range_vect));']);
            eval([ s '_power1_mean = zeros(size(range_vect));']);
            eval([ s '_power1_err = zeros(size(range_vect));']);
            eval([ s '_power2_mean = zeros(size(range_vect));']);
            eval([ s '_power2_err = zeros(size(range_vect));']);
            eval([ s '_power3_mean = zeros(size(range_vect));']);
            eval([ s '_power3_err = zeros(size(range_vect));']);
            for i_b=1:numel(range_vect)
                ran=range_vect(i_b);
                %%% value of the threshold
                vec=eval([ s '_list ( range_list == ran )']);
                eval([ s '_mean (i_b) = mean(vec);']);
                eval([ s '_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%% power of the threshold
                vec=eval([ s '_power_list ( range_list == ran )']);
                eval([ s '_power_mean (i_b) = mean(vec);']);
                eval([ s '_power_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%% number of steps (only interesting for step-down methods)
                vec=eval([ s '_nb_step_list ( range_list == ran )']);
                eval([ s '_nb_step_mean (i_b) = mean(vec);']);
                eval([ s '_nb_step_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%%
                %%% power of the threshold stopped at step 1
                vec=eval([ s '_power1_list ( range_list == ran )']);
                eval([ s '_power1_mean (i_b) = mean(vec);']);
                eval([ s '_power1_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%% power of the threshold stopped at step 2
                vec=eval([ s '_power2_list ( range_list == ran )']);
                eval([ s '_power2_mean (i_b) = mean(vec);']);
                eval([ s '_power2_err (i_b) = std(vec)/sqrt(numel(vec));']);
                %%% power of the threshold stopped at step 3
                vec=eval([ s '_power3_list ( range_list == ran )']);
                eval([ s '_power3_mean (i_b) = mean(vec);']);
                eval([ s '_power3_err (i_b) = std(vec)/sqrt(numel(vec));']);
            end
        end%for i_thr=1:(numel(thresholds_list))

        %%% Plot the data
        i_fig=i_fig+1;
        figure(i_fig);
        newplot;
        hold on;
        set(gca,'FontSize',fontsize);
        if z_error_bars
        %%% with error bars
            errorbar(range_vect,sd_quant_uncent_nb_step_mean,sd_quant_uncent_nb_step_err,'k-','Linewidth',linewidth);
            errorbar(range_vect,hybrid_nb_step_mean,hybrid_nb_step_err,'k--','Linewidth',linewidth);
        else
        %%% without error bars
            plot(range_vect,sd_quant_uncent_nb_step_mean,'k-','Linewidth',linewidth);
            plot(range_vect,hybrid_nb_step_mean,'k--','Linewidth',linewidth);
        end%if z_error_bars

        xlabel('Signal range (dB)');
        ylabel('Av. number of iterations');
        legend('s.d. quant uncent','hybrid approach','Location','NorthWest');
        hold off;

    
    %%%%% Plot data

    if z_eps
        print('-f','-r600','-depsc2',[print_dir 'Fig3_Left_' data_type '.eps']);
    end
    if z_pdf
        print('-f','-r600','-dpdf',[print_dir 'Fig3_Left_' data_type '.pdf']);
    end

    %%%% Figure 3 Right : Plot the average value of the power / average
    %%%% value of the maximum power
    i_fig=i_fig+1;
    figure(i_fig);
    newplot;
    hold on;
    set(gca,'FontSize',fontsize);
    if z_error_bars
    %%% with error bars
        errorbar(range_vect,sd_quant_uncent_power1_mean./sd_quant_uncent_power_mean,sd_quant_uncent_power1_err./sd_quant_uncent_power_mean,'k-.','Linewidth',linewidth);
        errorbar(range_vect,sd_quant_uncent_power2_mean./sd_quant_uncent_power_mean,sd_quant_uncent_power2_err./sd_quant_uncent_power_mean,'k--s','Linewidth',linewidth);
        errorbar(range_vect,sd_quant_uncent_power3_mean./sd_quant_uncent_power_mean,sd_quant_uncent_power3_err./sd_quant_uncent_power_mean,'k:s','Linewidth',linewidth);
        errorbar(range_vect,hybrid_power1_mean./hybrid_power_mean,hybrid_power1_err./hybrid_power_mean,'k-','Linewidth',linewidth);
        errorbar(range_vect,hybrid_power2_mean./hybrid_power_mean,hybrid_power2_err./hybrid_power_mean,'k--','Linewidth',linewidth);
        errorbar(range_vect,hybrid_power3_mean./hybrid_power_mean,hybrid_power3_err./hybrid_power_mean,'k:','Linewidth',linewidth);
    else
    %%% without error bars
        plot(range_vect,sd_quant_uncent_power1_mean./sd_quant_uncent_power_mean,'k-.','Linewidth',linewidth);
        plot(range_vect,sd_quant_uncent_power2_mean./sd_quant_uncent_power_mean,'k--s','Linewidth',linewidth);
        plot(range_vect,sd_quant_uncent_power3_mean./sd_quant_uncent_power_mean,'k:s','Linewidth',linewidth);
        plot(range_vect,hybrid_power1_mean./hybrid_power_mean,'k-','Linewidth',linewidth);
        plot(range_vect,hybrid_power2_mean./hybrid_power_mean,'k--','Linewidth',linewidth);
        plot(range_vect,hybrid_power3_mean./hybrid_power_mean,'k:','Linewidth',linewidth);
    end%if z_error_bars

    xlabel('Signal range (dB)');
    ylabel('Av. (power / max. power)');
    legend('sdqu, t=1','sdqu, t=2','sdqu, t=3','hybrid, t=1','hybrid, t=2','hybrid, t=3','Location','SouthWest');
    hold off;

    
    %%%%% Plot data

    if z_eps
        print('-f','-r600','-depsc2',[print_dir 'Fig3_Right_' data_type '.eps']);
    end
    if z_pdf
        print('-f','-r600','-dpdf',[print_dir 'Fig3_Right_' data_type '.pdf']);
    end

end%if z_fig3
