function W_mat=gener_Wmat(n,law,B)
% generates a matrix of random weights 'W_v' of size n*B with distribution given by 'law'
% possible values for the string law:
% 'Efr'    : Efron(n) weights
% 'Efr(M)' : Efron(M) weights
% 'Rho'    : Rho(n/2) weights (or (n-1)/2 if n is odd)
% 'Rho(q)' : Rho(q) weights
% 'Loo'    : Rho(n-1) weights = VFCV with V=n
% 'Rad'    : Rad(1/2) weights ( = +/- 1 ) 
% 'Rad(p)' : Rad(p) weights = Ber(p) weights - 1
% 'Ber'    : Ber(1/2) weights ( = 0 or 2 with equal probability ) = Rad(1/2) +1
% 'Ber(p)' : Ber(p) weights

%%%%%%% To be done:
% 'VFCV(V)' : V-fold cross-validation weights (the last argument, B, is not used)

switch law(1:3)
    case {'Ber','Rad'}
        if length(law)==3
            param=1/2;
        else
        	param=str2double(law(5:(length(law)-1)));
        end

        if (param>0 && param<1)
            W_mat=(1/param)*((rand(n,B)<param));
        else
            W_mat=NaN*ones(n,B);
        end
        
        if strcmp(law(1:3),'Rad')
            W_mat=W_mat - 1;
        end
    case {'VFC','Loo'}
        if strcmp(law,'Loo')
            param=n;
        else
            param=str2double(law(6:(length(law)-1)));
        end
        if (param>1 && param<=n && param==floor(param))
            list=1+floor((n/param)*(0:param));
            sig = randperm(n);%%% Randomize the order of the points for defining the blocks
            W_mat=(1+1/(param-1))*ones(n,param);
            for i=1:param
                W_mat(sig(list(i):(list(i+1)-1)),i)=zeros(list(i+1)-list(i),1);
            end
        else
            W_mat=NaN*ones(n,param);
        end
    otherwise
        W_mat = zeros(n,B);
        for i=1:B
            W_mat(:,i)=gener_W(n,law)';
        end
end

function W_v=gener_W(n,law)
% generates a random weight vector 'W_v' of size n with distribution given by 'law'
% possible values for the string law:
% 'Efr'    : Efron(n) weights
% 'Efr(M)' : Efron(M) weights
% 'Rho'    : Rho(n/2) weights (or (n-1)/2 if n is odd)
% 'Rho(q)' : Rho(q) weights
% 'Loo'    : Rho(n-1) weights
% 'Rad'    : Rad(1/2) weights
% 'Rad(p)' : Rad(p) weights

W_v = zeros(1,n);

switch law(1:3)
    case 'Efr'
        if length(law)==3
            param=n;
        else
            param=str2double(law(5:(length(law)-1)));
        end

        for i=1:param
            Uj = ceil(n*rand(1));
            W_v(Uj) = W_v(Uj) + (n/param);
        end

    case 'Rho'
	if length(law)==3
		param=floor(n/2);
	else
		param=min(n,str2double(law(5:(length(law)-1))));
	end

	sig = randperm(n);
	W_v(sig(1:param))=(n/param)+zeros(1,param);

    case 'Loo'
	W_v = (n/(n-1))*ones(1,n);
	W_v(ceil(n*rand(1)))=0;

    case 'Rad'
        if length(law)==3
            param=1/2;
        else
        	param=str2double(law(5:(length(law)-1)));
        end

    if (param>0 && param<1)
        W_v=(1/param)*((rand(1,n)<param));
    else
        W_v=NaN*ones(1,n);
    end

    otherwise
	W_v=NaN*ones(1,n);

end%switch law(1:3)
