function [opt, isdefault]= set_defaults(opt, varargin)
%[opt, isdefault]= set_defaults(opt, defopt)
%[opt, isdefault]= set_defaults(opt, field/value list)
%
% This functions fills in the given struct opt some new fields with
% default values, but only when these fields DO NOT exist before in opt.
% Existing fields are kept with their original values.
% There are two forms in which you can can specify the default values,
% (1) as struct, 
%   opt= set_defaults(opt, struct('color','g', 'linewidth',3));
%
% (2) as property/value list, e.g.,
%   opt= set_defaults(opt, 'color','g', 'linewidth',3);
%
% The second output argument isdefault is a struct with the same fields
% as the returned opt, where each field has a boolean value indicating
% whether or not the default value was inserted in opt for that field.

% fm (1) + bb (2), ida.first.fhg.de

if length(opt)>1,
  error('first argument must be a 1x1 struct');
end

isdefault= [];
if ~isempty(opt),
  for Fld=fieldnames(opt)',
    isdefault= setfield(isdefault, Fld{1}, 0);
  end
end

if length(varargin)==1,
  defopt = varargin{1};
  for Fld=fieldnames(defopt)',
    fld= Fld{1};
    if ~isfield(opt, fld),
      opt= setfield(opt, fld, getfield(defopt, fld));
      isdefault= setfield(isdefault, fld, 1);
    end
  end
  return
end

nArgs= length(varargin)/2;
if nArgs~=round(nArgs),
  error('inconsistent field/value list');
end

for ii= 1:nArgs,
  if ~isfield(opt, varargin{ii*2-1}),
    opt= setfield(opt, varargin{ii*2-1}, varargin{ii*2});
    isdefault= setfield(isdefault, varargin{ii*2-1}, 1);
  end
end
